<?php

namespace Drupal\ww_publish\Event;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\user\Entity\User;
use Drupal\ww_publish\Article;
use Drupal\ww_publish\Message;
use Drupal\ww_publish\Metadata;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\amazon_sns\Event\SnsEvents as AmazonSnsEvents;
use Drupal\amazon_sns\Event\SnsMessageEvent as AmazonSnsMessageEvent;

/**
 * Class SnsNotificationSubscriber.
 *
 * Subscribe to SNS notification events. These are in response to events
 * dispatched by NotificationController::receive(). This file is based on
 * \Drupal\amazon_sns\Event\SnsNotificationSubscriber.
 *
 * @package Drupal\ww_publish\Event
 */
class SnsNotificationSubscriber implements ContainerInjectionInterface, EventSubscriberInterface {

  /**
   * A logger instance.
   *
   * @var \Psr\Log\LoggerInterface
   */
  private $logger;

  /**
   * Configuration of the ww_publish module.
   *
   * @var \Drupal\Core\Config\Config
   */
  private $config;

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      AmazonSnsEvents::NOTIFICATION => 'createOrUpdateNode',
    ];
  }

  /**
   * SnsNotificationSubscriber constructor.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   A config factory.
   */
  public function __construct(LoggerChannelFactoryInterface $logger_factory, ConfigFactoryInterface $config_factory) {
    $this->logger = $logger_factory->get('ww_publish');
    $this->config = $config_factory->getEditable('ww_publish.settings');
    // Set user for logger.
    if ($userId = $this->config->get('ww_user')) {
      $currentUser = User::load($userId);
      $this->logger->setCurrentUser($currentUser);
    }
  }

  /**
   * {@inheritdoc}
   *
   * @codeCoverageIgnore
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('logger.factory'),
      $container->get('config.factory')
    );
  }

  /**
   * Create or update node.
   *
   * This will log any notifications received from SNS, regardless of type.  The
   * intention here is to help with tracking down SNS problems.
   *
   * @param \Drupal\amazon_sns\Event\SnsMessageEvent $event
   *   Publish message received from Amazon SNS.
   *
   * @return bool
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function createOrUpdateNode(AmazonSnsMessageEvent $event) {
    $eventMessage = $event->getMessage();

    if (isset($eventMessage['TopicArn']) && strpos($eventMessage['TopicArn'], 'ecs-export-topic' ) > 0 && isset($eventMessage['Message'])) {
      $eventMessageJson = json_decode($eventMessage['Message']);
      if ($this->config->get('debug_mode'))
        $this->logger->info('Amazon SNS message event: <pre><code>' . print_r($eventMessageJson, TRUE) . '</code></pre>');

      // Get article metadata.
      $message = new Message($eventMessageJson);
      $articleMetadata = $message->getArticleMetadata();
      $metadata = new Metadata($articleMetadata, $this->config, $this->logger);
      if ($this->config->get('debug_mode'))
        $this->logger->info('Article Metadata: <pre><code>' . print_r($metadata->getMetadata(), TRUE) . '</code></pre>');

      if ($metadata->validate()) {
        $articleData = $message->getArticleJSON();
        if ($this->config->get('debug_mode'))
          $this->logger->info('Article: <pre><code>' . print_r($articleData, TRUE) . '</code></pre>');
      } else {
        $this->logger->error('The metadata could not be validated.');
        return FALSE;
      }

      $preparedFiles = $message->prepareArticleFiles();
      if ($this->config->get('debug_mode'))
        $this->logger->info('Prepared files: <pre><code>' . print_r($preparedFiles, TRUE) . '</code></pre>');

      $article = new Article($metadata, $articleData, $preparedFiles, $this->config, $this->logger);
      $article->publish();

    } else {
      $this->logger->error('Amazon SNS Message event does not contain expected properties.');
      return FALSE;
    }

    return TRUE;
  }
}
